import { OrderedSet, List } from "immutable";
import { TargetLanguage } from "../TargetLanguage";
import { Type, ClassType, EnumType, UnionType } from "../Type";
import { TypeGraph } from "../TypeGraph";
import { Name, Namer } from "../Naming";
import { Sourcelike } from "../Source";
import { ConvenienceRenderer, ForbiddenWordsInfo } from "../ConvenienceRenderer";
import { Option } from "../RendererOptions";
import { Declaration } from "../DeclarationIR";
export declare type NamingStyle = "pascal" | "camel" | "underscore" | "upper-underscore";
export default class CPlusPlusTargetLanguage extends TargetLanguage {
    private readonly _justTypesOption;
    private readonly _namespaceOption;
    private readonly _typeNamingStyleOption;
    private readonly _memberNamingStyleOption;
    private readonly _enumeratorNamingStyleOption;
    constructor();
    protected getOptions(): Option<any>[];
    readonly supportsUnionsWithBothNumberTypes: boolean;
    protected readonly rendererClass: new (targetLanguage: TargetLanguage, graph: TypeGraph, leadingComments: string[] | undefined, ...optionValues: any[]) => ConvenienceRenderer;
}
export declare class CPlusPlusRenderer extends ConvenienceRenderer {
    private readonly _justTypes;
    private readonly _namespaceNames;
    private readonly _typeNameStyle;
    private readonly _typeNamingFunction;
    private readonly _memberNamingFunction;
    private readonly _caseNamingFunction;
    constructor(targetLanguage: TargetLanguage, graph: TypeGraph, leadingComments: string[] | undefined, _justTypes: boolean, namespaceName: string, _typeNamingStyle: NamingStyle, _memberNamingStyle: NamingStyle, _enumeratorNamingStyle: NamingStyle);
    protected forbiddenNamesForGlobalNamespace(): string[];
    protected forbiddenForObjectProperties(_c: ClassType, _className: Name): ForbiddenWordsInfo;
    protected forbiddenForEnumCases(_e: EnumType, _enumName: Name): ForbiddenWordsInfo;
    protected makeNamedTypeNamer(): Namer;
    protected namerForObjectProperty(): Namer;
    protected makeUnionMemberNamer(): null;
    protected makeEnumCaseNamer(): Namer;
    protected readonly needsTypeDeclarationBeforeUse: boolean;
    protected canBeForwardDeclared(t: Type): boolean;
    protected emitDescriptionBlock(lines: string[]): void;
    protected emitBlock: (line: Sourcelike, withSemicolon: boolean, f: () => void, withIndent?: boolean) => void;
    protected emitNamespaces: (namespaceNames: List<string>, f: () => void) => void;
    protected cppTypeInOptional: (nonNulls: OrderedSet<Type>, ctx: {
        needsForwardIndirection: boolean;
        needsOptionalIndirection: boolean;
        inJsonNamespace: boolean;
    }, withIssues: boolean) => Sourcelike;
    protected variantType: (u: UnionType, inJsonNamespace: boolean) => Sourcelike;
    protected ourQualifier: (inJsonNamespace: boolean) => Sourcelike;
    protected jsonQualifier: (inJsonNamespace: boolean) => Sourcelike;
    protected variantIndirection: (needIndirection: boolean, typeSrc: Sourcelike) => Sourcelike;
    protected cppType: (t: Type, ctx: {
        needsForwardIndirection: boolean;
        needsOptionalIndirection: boolean;
        inJsonNamespace: boolean;
    }, withIssues: boolean) => Sourcelike;
    protected emitClassMembers: (c: ClassType) => void;
    protected emitClass: (c: ClassType, className: Name) => void;
    protected emitClassFunctions: (c: ClassType, className: Name) => void;
    protected emitEnum: (e: EnumType, enumName: Name) => void;
    protected emitUnionTypedefs: (u: UnionType, unionName: Name) => void;
    protected emitUnionFunctions: (u: UnionType) => void;
    protected emitEnumFunctions: (e: EnumType, enumName: Name) => void;
    protected emitTopLevelTypedef: (t: Type, name: Name) => void;
    protected emitAllUnionFunctions: () => void;
    protected emitOptionalHelpers: () => void;
    protected emitDeclaration(decl: Declaration): void;
    protected emitTypes: () => void;
    protected emitSourceStructure(): void;
}
